# -*- coding: utf-8 -*-

'''
set of functions that handle your watchlist and theaters files.
'''

import os.path

DATA_PATH = "data/"
WATCHLIST_FILE = "watchlist.txt"
THEATERS_FILE = "theaters.txt"
MY_PATH = os.path.abspath(os.path.dirname(__file__))


# --------------------- #
# watchlist helpers     #
# --------------------- #
def get_watchlist() -> set:
    '''Reads the watchlist file and returns a set of movies with titles lowercased.'''
    watchlist = os.path.join(MY_PATH, f"{DATA_PATH}{WATCHLIST_FILE}")
    assert os.path.exists(watchlist), f"{watchlist} file does not exist."
    with open(watchlist) as f:
        return frozenset((x[:-1]).lower() for x in f.readlines())

# --------------------- #
# theater helpers       #
# --------------------- #
def get_theaters() -> list:
    '''Reads the theaters file and returns a set of theaters.'''
    theaterlist = os.path.join(MY_PATH, f"{DATA_PATH}{THEATERS_FILE}")
    assert os.path.exists(theaterlist), f"{theaterlist} file does not exist."
    with open(theaterlist) as f:
        return f.readlines()

def add_theater(theater_name):
    '''Adds a theater to your favorite theater file'''
    theaterlist = os.path.join(MY_PATH, f"{DATA_PATH}{THEATERS_FILE}")
    assert os.path.exists(theaterlist), f"{theaterlist} file does not exist."
    with open(theaterlist) as f:
        f.write(theater_name)

def remove_theater(theater_name):
    '''Remove theater from you list of favorite theaters'''
    pass



'''
run point of application. Calls films to cal module.
Will soon allow configuration and downloading of watchlist.
'''
import configparser
import argparse
import os
from film_to_cal import film_to_cal

# get default settings from settings.ini file
THIS_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
SETTINGS_FILE = "settings.ini"
config = configparser.ConfigParser()
config.read(f"{THIS_DIRECTORY}/data/{SETTINGS_FILE}")
settings = config['DEFAULT']

# TYPE VERIFICATION
def valid_timezone(timezone_input):
    '''checks whether input string is valid IANA timezone'''
    TIMEZONE_FILE = "timezones.txt"
    with open(f"{THIS_DIRECTORY}/data/{TIMEZONE_FILE}") as f:
        timezones = frozenset([line[:-1] for line in f.readlines()])
    if timezone_input not in timezones:
        msg = f"{string} is not a IANA timezone"
        raise argparse.ArgumentTypeError(msg)
    return timezone_input

def valid_workdays(workdays_input):
    '''checks whether string is valid workday'''
    msg = f"{workdays_input} is not a valid workdays input. Your input should be a subset of [0,1,2,3,4,5,6] presented as a comma seperated string. ex: 1,2,4."
    try:
        workdays = set(map(int, workdays_input.split(',')))
    except:
        raise argparse.ArgumentTypeError(msg)
    if workdays.issubset(set(range(6))):
        return workdays
    raise argparse.ArgumentTypeError(msg)

def valid_workhours(workhours_input):
    '''checks whether string is valid workhour'''
    msg = f"{workhours_input} is not a valid workhours input. Your input should be two comma-seperated integers between 0 and 23 inclusive. ex. (10,20). These designate the start of work hours and the end of work hours"
    try:
        work_start, work_end = map(int, workhours_input.split(","))
    except:
        raise argparse.ArgumentTypeError(msg)
    if 23>work_start>0 and 23>work_end>0:
        return ((work_start,0), (work_end,0))
    raise argparse.ArgumentTypeError(msg)

def str2bool(v):
    if isinstance(v, bool):
       return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')

def set_up_argparse():
    '''SET UP ARGPARSE'''
    parser = argparse.ArgumentParser(description='Discover which movies in your watchlist are playing in your favorite theaters. You can change defaults by updating the data/settings.ini file.')

    parser.add_argument('-b', '--browser', default=settings['browser'], choices=['firefox', 'chrome'], help=f"Whether you want to run a chrome or firefox headless browser (default: {settings['browser']}).")

    parser.add_argument('-a', '--all-films', type=str2bool, nargs='?', const=True, default=False, help=f"If you want all films instead of filtering based on your watchlist.txt file. (default: False).")

    parser.add_argument('-d', '--workdays', type=valid_workdays, default=settings['workdays'], help=f"Which days you work. A comma seperated list of numbers between 0 and 6 (0 is monday, 1 is tueday, ..., 6 is sunday). (default: {settings['workdays']}).")

    parser.add_argument('-w', '--workhours', type=valid_workhours, default=settings['workhours'], help=f"Which hours do you work. Hours are in 24 hour time and formatted: start_hour,end_hour. (default: {settings['workhours']}).")

    parser.add_argument('-t', '--timezone', type=valid_timezone, default=settings['timezone'], help=f"Make sure you provide a valid IANA timezone. Used by the google calendar api. (default: {settings['timezone']}).")

    return vars(parser.parse_args())

def main():
    args = set_up_argparse()
    print(args)
    film_to_cal(args)

# MAIN FUNCTION: CALLS FILM TO CAL
if __name__ == "__main__":
    main()
    '''
     TODO #
    - add favorite theater
    - update watchlist using letterboxd
    - set busy schedule
    - keep track of films already scheduled and do not prompt if sceance has passed
    '''



# -*- coding: utf-8 -*-

'''
setup.py file for sceance
'''

from setuptools import setup, Extension
import sys

if sys.version_info[:3] < (3, 0, 0):
    print("Requires Python 3 to run.")
    sys.exit(1)

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(
    name='sceance',
    author='Samuel Mignot',
    author_email='sjmignot@gmail.com',
    version='0.2.0.1',
    description='Uses a watchlist and favorite theaters to suggest film screenings.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/sjmignot/sceance',
    download_url='https://github.com/sjmignot/sceance/archive/v_01.tar.gz',
    packages=['sceance'],
    package_dir={'sceance': 'sceance'},
    package_data={'sceance': ['data/*.txt', 'data/*.ini']},
    keywords = ['movie theater', 'showtimes', 'selenium', 'movies', 'film'],
    install_requires=[
        'requests>2',
        'google_api_python_client>1',
        'google_auth_oauthlib',
        'pandas',
        'protobuf>3',
        'selenium>3',
        'webdriver_manager>2',
        'progress>1',
    ],
    entry_points={
        'console_scripts': [
            'sceance=sceance.__main__:main',
        ]
    },
    liscence="MIT",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License'
    ]
)



# -*- coding: utf-8 -*-

'''
uses the get_showtime_watchlist module to get all film showings in watchlist during work week.
Offers to create calender events for each movie at free times in calender that aren't during
the work week.

Using google calender's API.
'''

import os
import os.path

import datetime
import pickle

from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request

# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/calendar']

TOKEN_FILE = "token.pickle"
CRED_FILE = "credentials.json"

MY_PATH = os.path.abspath(os.path.dirname(__file__))
DATA_PATH = 'data/'

def build_and_add_event(service, film, theater, showtime, timezone):
    '''builds an event object and adds it to the google calendar'''
    start_time = showtime
    end_time = start_time+datetime.timedelta(hours=int(film.length[0]), minutes=int(film.length[1]))

    event = {
        'summary': f'{film.name} at {theater.name}',
        'location': theater.address,
        'start': {
            'dateTime': start_time.isoformat(),
            'timeZone': timezone,
        },
        'end': {
            'dateTime': end_time.isoformat(),
            'timeZone': timezone,
        },
    }

    event = service.events().insert(calendarId='primary', body=event).execute()
    print('Event created: %s' % (event.get('htmlLink')))

def get_creds():
    '''gets and returns user credentials either from a token or creates a new token if said token expired'''
    creds = None
    token_file = os.path.join(MY_PATH, f"{DATA_PATH}{TOKEN_FILE}")
    if os.path.exists(token_file):
        with open(token_file, 'rb') as token:
            creds = pickle.load(token)

    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                os.path.join(MY_PATH, f"{DATA_PATH}{CRED_FILE}"), SCOPES
            )
            creds = flow.run_local_server(port=0)

        # Save the credentials for the next run
        with open(token_file, 'wb') as token:
            pickle.dump(creds, token)
    return creds

def create_projection_events(projection_list, timezone):
    '''gets credentials, builds calendar API object and for each projection, builds and sends an event to your calendar'''
    creds = get_creds()
    service = build('calendar', 'v3', credentials=creds)
    for project in projection_list:
        print(project)
        build_and_add_event(service, *project, timezone)
